import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import org.htmlparser.NodeFilter;
import org.htmlparser.Parser;
import org.htmlparser.filters.NodeClassFilter;
import org.htmlparser.filters.NotFilter;
import org.htmlparser.filters.OrFilter;
import org.htmlparser.filters.TagNameFilter;
import org.htmlparser.tags.AppletTag;
import org.htmlparser.tags.FrameSetTag;
import org.htmlparser.tags.FrameTag;
import org.htmlparser.tags.MetaTag;
import org.htmlparser.tags.ObjectTag;
import org.htmlparser.tags.ScriptTag;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;


public class htmlfilter {

	/**
	 * This html filter removes certain insecure html tags. The list of tags that will be removed are:
	 * <applet> prevent java applet from loading
	 * <frame> prevent loading of external urls that could potentially contain harmful contents such as scripts or flash
	 * <frameset> same as above
	 * <iframe> same as above
	 * <meta> prevent http-equiv such as "refresh" or "cookie", since setting cookie on the user's computer does leave an permanent effect
	 * <object> prevent flash, active-X, java applet, etc
	 * <script> prevent client side scripts
	 * 
	 * This html filter uses open source library "htmlparser", available at http://htmlparser.sourceforge.net/
	 */
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		/**
		 * This function first read the unsafe html from stdin, clean it, then write it to stdout.
		 */
		String htmlText = readHtml();
		String cleanText = sanitize(htmlText);
		writeHtml(cleanText);
	}
	

	
	public static String sanitize(String text) {
		/**
		 * The main filter function. Removes all nodes of question. 
		 */
		Parser parser = new Parser();
		NodeList fullList = new NodeList();
		String cleanText = "";
		NodeFilter[] filterList = {
				new NodeClassFilter (ScriptTag.class),
				new NodeClassFilter (AppletTag.class),
				new NodeClassFilter (FrameTag.class),
				new NodeClassFilter (FrameSetTag.class),
				new NodeClassFilter (MetaTag.class),
				new NodeClassFilter (ObjectTag.class),
				new TagNameFilter("iframe")
		};
		NodeFilter filter = new OrFilter (filterList);
		filter = new NotFilter (filter);
		try {
			parser.setInputHTML(text);
			fullList = parser.parse(null);
			fullList.keepAllNodesThatMatch(filter, true);
			cleanText=fullList.toHtml();
		
		} catch (ParserException e) {
			//e.printStackTrace();
			System.out.println("Sorry, your html is not properly formated. We will show a blank page instead. Go learn html first!");
		}
		return cleanText;
	}
	
	
	
	
	public static String readHtml() {
		/**
		 * read the unsafe html from stdin
		 */
		String result = "";
		try{
			InputStreamReader input = new InputStreamReader(System.in);
			BufferedReader br = new BufferedReader(input);
			String strLine;
			while ((strLine = br.readLine()) != null)   {
				result = result+strLine;
			}
			input.close();
		}catch (Exception e){
		  System.err.println("Stdin reading error: " + e.getMessage());
		}
		return result;
	}
	
	public static boolean writeHtml (String cleanText) {
		/**
		 * Write the clean html to stdout
		 */
		try{
			OutputStreamWriter output = new OutputStreamWriter(System.out);
    	    BufferedWriter out = new BufferedWriter(output);
    	    out.write(cleanText);
    	    out.close();
		}catch (Exception e){
			System.err.println("stdout writing error: " + e.getMessage());
			return false;
		}
		return true;
	}
	
}
